/* --COPYRIGHT--,BSD
 * Copyright (C) 2019 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * --/COPYRIGHT--*/
#include <msp430.h>
#include "main.h"
#include "stdio.h"

USS_Algorithms_Results algorithms_Results;

static void checkCode(USS_message_code code, USS_message_code expectedCode);
static void handlePllUnlockEvent(void);
static void disableApplicationInterrupts(void);
static void enableApplicationInterrupts(void);
static void start_conversion_ISR(void);
static void data_ready_ISR(void);
static void ping_done_ISR(void);
static void sequence_done_ISR(void);

#if APPLICATION_ENABLE_UART_DEBUG
static void uartTxByte(char string);
static void uartTxString(char *buf);
static void uartTxHexByte(uint8_t Data);
static uint8_t nibbleToHex(uint8_t nibble);
static void uartTxUSSResult(uint8_t delimiter, float *data);
#endif

int main(void)
{
#if (APPLICATION_ENABLE_CHANNEL_SWAP == true)
    uint16_t appSwapInterval = APPLICATION_CHANNEL_SWAP_INTERVAL;
#endif

#if (APPLICATION_ENABLE_ABSTOF_DTOF_OFFSET_CALIBRATION == true)
    USS_dTof_absTof_offset_results abstoFDtofTestResults;
    USS_dTof_absTof_offset_test_config abstoFDtofTestConfig =
    {
         .numOfTestIterations        = APPLICATION_ABSTOF_DTOF_OFFSET_UPDATE_INTERVAL,
         .isUseLPMCapture            = APPLICATION_ABSTOF_DTOF_OFFSET_LPM_CAPTURE,
         .isCalculateUpsAbsTofOffset = APPLICATION_ABSTOF_DTOF_OFFSET_CALC_UPS_ABSTOF,
         .isCalculateDnsAbsTofOffset = APPLICATION_ABSTOF_DTOF_OFFSET_CALC_DNS_ABSTOF,
         .isCalculateDToFOffset      = APPLICATION_ABSTOF_DTOF_OFFSET_CALC_DTOF,
    };
#endif

    volatile USS_message_code code;
    USS_Algorithms_Results algResults;
    USS_calibration_hspll_results testResults;


    // Register PLL unlock event
    USS_registerHSPLLInterruptCallback(USS_HSPLL_Interrupt_PLLUNLOCK,
                                       &handlePllUnlockEvent);

    code = USS_configureUltrasonicMeasurement(&gUssSWConfig);
    checkCode(code, USS_message_code_no_error);

#if((USS_ALG_ABS_TOF_COMPUTATION_MODE == USS_ALG_ABS_TOF_COMPUTATION_MODE_LOBE_WIDE) || \
    (USS_ALG_ABS_TOF_COMPUTATION_MODE == USS_ALG_ABS_TOF_COMPUTATION_MODE_HILBERT_WIDE))
    // Reference binary pattern are only needed by
    // USS_Alg_AbsToF_Calculation_Option_lobeWide and
    // USS_Alg_AbsToF_Calculation_Option_hilbertWide AbsToF computation options
    if((USS_Alg_AbsToF_Calculation_Option_lobeWide ==
            gUssSWConfig.algorithmsConfig->absToFOption)
       || (USS_Alg_AbsToF_Calculation_Option_hilbertWide ==
               gUssSWConfig.algorithmsConfig->absToFOption))
    {
#if defined(__MSP430_HAS_SAPH_A__)
        if(USS_measurement_pulse_generation_mode_multi_tone ==
                gUssSWConfig.measurementConfig->pulseConfig->pulseGenMode)
        {
            code = USS_generateMultiToneBinaryPattern(&gUssSWConfig);
            checkCode(code, USS_message_code_no_error);
        }
#endif
        if(USS_measurement_pulse_generation_mode_multi_tone !=
                gUssSWConfig.measurementConfig->pulseConfig->pulseGenMode)
        {
            code = USS_generateMonoDualToneBinaryPattern(&gUssSWConfig);
            checkCode(code, USS_message_code_no_error);
        }
    }

#if (APPLICATION_ENABLE_BINARY_PATTERN_SIZE_SCALING == true)
    gUssSWConfig.algorithmsConfig->binaryPatternLength =
            (gUssSWConfig.captureConfig->sampleSize / APPLICATION_BINARY_PATTERN_SCALE_FACTOR);
#endif

#endif


    // Application must ensure no application level interrupts occur while
    // verifying HSPLL Frequency
    disableApplicationInterrupts();

    code = USS_verifyHSPLLFrequency(&gUssSWConfig, &testResults);

    // Application can re-enable interrupts after HSPLL verification
    enableApplicationInterrupts();

    checkCode(code, USS_message_code_no_error);

    gUssSWConfig.algorithmsConfig->clockRelativeError = _IQ27div((int32_t)(testResults.actualTestCount -
            testResults.expectedResult),testResults.expectedResult);

    code = USS_initAlgorithms(&gUssSWConfig);
    checkCode(code, USS_message_code_no_error);

#if (APPLICATION_ENABLE_SIGNAL_GAIN_CALIBRATION == true)
    code = USS_calibrateSignalGain(&gUssSWConfig);
    checkCode(code, USS_message_code_Signal_Gain_Calibration_successful);
#endif

#if (APPLICATION_ENABLE_ABSTOF_DTOF_OFFSET_CALIBRATION == true)
    code = USS_calculateOffsets(&gUssSWConfig, &abstoFDtofTestResults,
                                &abstoFDtofTestConfig);
    checkCode(code, USS_message_code_no_error);

    code = USS_updateAdditionalCaptureDelay(&gUssSWConfig,
          ((abstoFDtofTestResults.upsAbsToFOffset + abstoFDtofTestResults.dnsAbsToFOffset) /2.0f) -
          APPLICATION_ABSTOF_REFERENCE);
    checkCode(code, USS_message_code_no_error);

    code = USS_updateDtoFOffset(&gUssSWConfig, (-1.0f *abstoFDtofTestResults.dToFOffset));
    checkCode(code, USS_message_code_no_error);

#endif

    // Set the background timer period to 1 second
    USS_configAppTimerPeriod(&gUssSWConfig, gUssSWConfig.systemConfig->measurementPeriod);  //timer1, defined by (USS_LFXT_FREQ_IN_HZ/USS_SYS_MEASUREMENT_PERIOD)

    P7SEL0 &= ~(1<<5);    //set GPIO
    P7SEL1 &= ~(1<<5);    //set GPIO
    P7DIR |= (1<<5);    //set output

    USS_registerSDHSInterruptCallback(USS_SDHS_Interrupt_SSTRG, start_conversion_ISR);

    USS_registerSDHSInterruptCallback(USS_SDHS_Interrupt_DTRDY, data_ready_ISR);

    USS_registerSAPHInterruptCallback(USS_SAPH_Interrupt_PNGDN, ping_done_ISR);

    USS_registerSAPHInterruptCallback(USS_SAPH_Interrupt_SEQDN, sequence_done_ISR);

    while(1)
    {

        P7OUT |= (1<<5);    //set high
        //code = USS_startLowPowerUltrasonicCapture(&gUssSWConfig);
        code = USS_startUltrasonicMeasurement(&gUssSWConfig, USS_capture_power_mode_active);
        checkCode(code, USS_message_code_no_error);
        P7OUT &= ~(1<<5);   //set low
        code = USS_runAlgorithms(&gUssSWConfig,&algResults);
        checkCode(code, USS_message_code_valid_results);

#if (APPLICATION_ENABLE_CHANNEL_SWAP == true)
        if(appSwapInterval == 0)
        {
            code = USS_swapCaptureChannels(&gUssSWConfig);
            code = USS_swapAlgorithmsCaptureBuffers(&gUssSWConfig);

            appSwapInterval = APPLICATION_CHANNEL_SWAP_INTERVAL;
        }else{
            appSwapInterval--;
        }
#endif

#if APPLICATION_ENABLE_UART_DEBUG
        char buf[100];
        //compiler flag changed from --printf_support=minimal to --printf_support=full
        float distance_mm = ((algResults.totalTOF_UPS+algResults.totalTOF_DNS)/4)*1481*1000;
        sprintf(buf, "/*%f,%f,%f,%f*/",algResults.totalTOF_UPS*1000000.0,algResults.totalTOF_DNS*1000000.0,21.0,distance_mm);
        uartTxString(buf);


#endif
        // Wait for timer to elapse
        USS_waitForAppTimerElapse(&gUssSWConfig,USS_low_power_mode_option_low_power_mode_3);
    }
}

#if APPLICATION_ENABLE_UART_DEBUG

void uartTxByte(char string)
{
    while(!(UCA1IFG & UCTXIFG));
    UCA1TXBUF = string;
}

void uartTxString(char *buf){
    uint8_t i = 0;

    while(buf[i] != '\0'){
        uartTxByte(buf[i]);
        i++;
    }
}

void uartTxHexByte(uint8_t Data)
{
    uartTxByte(nibbleToHex(Data >> 4));
    uartTxByte(nibbleToHex(Data & 0x0F));
}

uint8_t nibbleToHex(uint8_t nibble)
{
    uint8_t ret = nibble;
    if(nibble < 10)
    {
        ret += '0';
    }
    else
    {
        ret += 'A' - 10;
    }

    return(ret);
}

#endif

void handlePllUnlockEvent(void)
{
    // If USS PLL unlock event is detected rest USS Module and reconfigure
    // measurement
    USS_resetUSSModule(&gUssSWConfig, true);
}


// This is a place holder for the application to disable interrupts which might
// be triggered during HSPLL verification
void disableApplicationInterrupts(void){

}

// This is a place holder for the application to enable interrupts once
// HSPLL verification has completed
void enableApplicationInterrupts(void){

}

void checkCode(USS_message_code code, USS_message_code expectedCode)
{
    if(code != expectedCode)
    {
        // Trap code
        while(1);
    }
}

void start_conversion_ISR(void){
    P7OUT &= ~(1<<5);   //set low
    P7OUT |= (1<<5);    //set high
}

void data_ready_ISR(void){
    P7OUT &= ~(1<<5);   //set low
    P7OUT |= (1<<5);    //set high
    P7OUT &= ~(1<<5);   //set low
    P7OUT |= (1<<5);    //set high
}

void ping_done_ISR(void){
    P7OUT &= ~(1<<5);   //set low
    P7OUT |= (1<<5);    //set high
    P7OUT &= ~(1<<5);   //set low
    P7OUT |= (1<<5);    //set high
    P7OUT &= ~(1<<5);   //set low
    P7OUT |= (1<<5);    //set high
}

void sequence_done_ISR(void){
    P7OUT &= ~(1<<5);   //set low
    P7OUT |= (1<<5);    //set high
    P7OUT &= ~(1<<5);   //set low
    P7OUT |= (1<<5);    //set high
    P7OUT &= ~(1<<5);   //set low
    P7OUT |= (1<<5);    //set high
    P7OUT &= ~(1<<5);   //set low
    P7OUT |= (1<<5);    //set high
}
